@@ -27,6 +27,9 @@ class Node:
     def has_outgoing_links(self):
         return len(self.outgoing_link_list) > 0
 
+    def has_incoming_links(self):
+        return len(self.incoming_link_list) > 0
+
     def get_zone_id(self):
         return self.zone_id
 
@@ -441,6 +444,10 @@ class Network:
         agent_no = agent_id - 1
         agent = self._get_agent(agent_no)
 
+        path_cost = agent.get_path_cost()
+        if path_cost >= MAX_LABEL_COST:
+            return f'distance: infinitity | path: '
+
         path = ''
         if agent.node_path:
             path = ';'.join(
@@ -450,7 +457,7 @@ class Network:
         if path_only:
             return path
         else:
-            return f'distance: {agent.get_path_cost():.2f} | node path: {path}'
+            return f'distance: {path_cost:.2f} | node path: {path}'
 
     def get_agent_link_path(self, agent_id, path_only):
         """ return the sequence of link IDs along the agent path
@@ -461,6 +468,10 @@ class Network:
         agent_no = agent_id - 1
         agent = self._get_agent(agent_no)
 
+        path_cost = agent.get_path_cost()
+        if path_cost >= MAX_LABEL_COST:
+            return f'distance: infinitity | path: '
+        
         path = ''
         if agent.link_path:
             path = ';'.join(
@@ -470,7 +481,7 @@ class Network:
         if path_only:
             return path
         else:
-            return f'distance: {agent.get_path_cost():.2f} | link path: {path}'
+            return f'distance: {path_cost:.2f} | link path: {path}'
 
     def get_agent_orig_node_id(self, agent_id):
         """ return the origin node id of agent """
@@ -510,6 +521,9 @@ class Network:
     def get_zones(self):
         return self.zones
 
+    def get_zone_size(self):
+        return len(self.zone_to_nodes_dict)
+
     def get_from_node_no_arr(self):
         return self.from_node_no_array
 
