@@ -1,5 +1,5 @@
-import csv
 import os
+import csv
 import threading
 
 from .classes import Node, Link, Network, Column, ColumnVec, VDFPeriod, \
@@ -18,6 +18,58 @@ __all__ = [
 ]
 
 
+# for precheck on connectivity of each OD pair
+# 0: isolated, has neither outgoing links nor incoming links
+# 1: has at least one outgoing link
+# 2: has at least one incoming link
+# 3: has both outgoing and incoming links
+_zone_degrees = []
+
+
+def _initialize_zone_degrees(zone_size):
+    assert(zone_size > 0)
+
+    global _zone_degrees
+    _zone_degrees = [0] * zone_size
+
+
+def _update_orig_zone(oz_id):
+    global _zone_degrees
+
+    if _zone_degrees[oz_id] == 0:
+        _zone_degrees[oz_id] = 1
+    elif _zone_degrees[oz_id] == 2:
+        _zone_degrees[oz_id] = 3
+
+
+def _update_dest_zone(dz_id):
+    global _zone_degrees
+
+    if _zone_degrees[dz_id] == 0:
+        _zone_degrees[dz_id] = 2
+    elif _zone_degrees[dz_id] == 1:
+        _zone_degrees[dz_id] = 3
+
+
+def _are_od_connected(oz_id, dz_id):
+    connected = True
+
+    # at least one node in O must have outgoing links
+    if _zone_degrees[oz_id] == 0 or _zone_degrees[oz_id] == 2:
+        connected = False
+        print(f'WARNING! {oz_id} has no outgoing links to route volume '
+              f'between OD: {oz_id} --> {dz_id}')
+
+    # at least one node in D must have incoming links
+    if _zone_degrees[dz_id] == 0 or _zone_degrees[dz_id] == 1:
+        if connected:
+            connected = False
+        print(f'WARNING! {dz_id} has no incoming links to route volume '
+              f'between OD: {oz_id} --> {dz_id}')
+
+    return connected
+
+
 def _convert_str_to_int(str):
     """
     TypeError will take care the case that str is None
@@ -133,12 +185,12 @@ def read_nodes(input_dir,
         for line in reader:
             # set up node_id, which should be an integer
             node_id = _convert_str_to_int(line['node_id'])
-            if not node_id:
+            if node_id is None:
                 continue
 
             # set up zone_id, which should be an integer
             zone_id = _convert_str_to_int(line['zone_id'])
-            if not zone_id:
+            if zone_id is None:
                 zone_id = -1
 
             # treat them as string
@@ -175,13 +227,18 @@ def read_links(input_dir,
                nodes,
                id_to_no_dict,
                link_id_dict,
+               zone_size,
                agent_type_size,
-               demand_period_size):
+               demand_period_size,
+               load_demand):
 
     """ step 2: read input_link """
     with open(input_dir+'/link.csv', 'r', encoding='utf-8') as fp:
         print('read link.csv')
 
+        if load_demand:
+            _initialize_zone_degrees(zone_size)
+
         reader = csv.DictReader(fp)
         link_seq_no = 0
         for line in reader:
@@ -190,15 +247,15 @@ def read_links(input_dir,
 
             # check the validility
             from_node_id = _convert_str_to_int(line['from_node_id'])
-            if not from_node_id:
+            if from_node_id is None:
                 continue
 
             to_node_id =_convert_str_to_int(line['to_node_id'])
-            if not to_node_id:
+            if to_node_id is None:
                 continue
 
             length = _convert_str_to_float(line['length'])
-            if not length:
+            if length is None:
                 continue
 
             # pass validility check
@@ -215,20 +272,20 @@ def read_links(input_dir,
             # if they are not None, convert them to the corresponding types
             # if they are None's, set them using the default values
             lanes = _convert_str_to_int(line['lanes'])
-            if not lanes:
+            if lanes is None:
                 lanes = 1
 
             link_type = _convert_str_to_int(line['link_type'])
-            if not link_type:
+            if link_type is None:
                 link_type = 1
 
             free_speed = _convert_str_to_int(line['free_speed'])
-            if not free_speed:
+            if free_speed is None:
                 free_speed = 60
 
             # issue: int??
             capacity = _convert_str_to_int(line['capacity'])
-            if not capacity:
+            if capacity is None:
                 capacity = 49500
 
             # if link.csv does not have no column 'allowed_uses',
@@ -349,10 +406,19 @@ def read_links(input_dir,
                 link.vdfperiods.append(vdf)
 
             # set up outgoing links and incoming links
-            nodes[from_node_no].add_outgoing_link(link)
-            nodes[to_node_no].add_incoming_link(link)
+            from_node = nodes[from_node_no]
+            to_node = nodes[to_node_no]
+            from_node.add_outgoing_link(link)
+            to_node.add_incoming_link(link)
             links.append(link)
 
+            # set up zone degrees
+            if load_demand:
+                oz_id = from_node.get_zone_id()
+                dz_id = to_node.get_zone_id()
+                _update_orig_zone(oz_id)
+                _update_dest_zone(dz_id)
+
             link_seq_no += 1
 
         print(f"the number of links is {link_seq_no}")
@@ -377,12 +443,12 @@ def read_demand(input_dir,
         for line in reader:
             # invalid origin zone id, discard it
             oz_id = _convert_str_to_int(line['o_zone_id'])
-            if not oz_id:
+            if oz_id is None:
                 continue
 
             # invalid destinationzone id, discard it
             dz_id = _convert_str_to_int(line['d_zone_id'])
-            if not dz_id:
+            if dz_id is None:
                 continue
 
             # o_zone_id does not exist in node.csv, discard it
@@ -394,7 +460,14 @@ def read_demand(input_dir,
                 continue
 
             volume = _convert_str_to_float(line['volume'])
-            if not volume:
+            if volume is None:
+                continue
+
+            if volume == 0:
+                continue
+
+            # precheck on connectivity of each OD pair
+            if not _are_od_connected(oz_id, dz_id):
                 continue
 
             # set up volume for ColumnVec
@@ -402,12 +475,12 @@ def read_demand(input_dir,
                 column_pool[(at, dp, oz_id, dz_id)] = ColumnVec()
             column_pool[(at, dp, oz_id, dz_id)].od_vol += volume
 
-            if volume == 0:
-                continue
-
             total_agents += int(volume + 1)
 
-    print(f"the number of agents is {total_agents}")
+        print(f"the number of agents is {total_agents}")
+
+        if total_agents == 0:
+            raise Exception('NO VALID OD VOLUME!! DOUBLE CHECK YOUR demand.csv')
 
 
 def _auto_setup(assignment):
@@ -505,8 +578,10 @@ def read_network(load_demand='true', input_dir='.'):
                network.node_list,
                network.internal_node_seq_no_dict,
                network.link_id_dict,
+               network.get_zone_size(),
                assignm.get_agent_type_count(),
-               assignm.get_demand_period_count())
+               assignm.get_demand_period_count(),
+               load_demand)
 
     if load_demand:
         for d in assignm.get_demands():
@@ -544,24 +619,24 @@ def load_columns(ui, input_dir='.'):
         for line in reader:
             # critical info
             oz_id = _convert_str_to_int(line['o_zone_id'])
-            if not oz_id:
+            if oz_id is None:
                 continue
 
             dz_id = _convert_str_to_int(line['d_zone_id'])
-            if not dz_id:
+            if dz_id is None:
                 continue
 
             node_seq = line['node_sequence']
-            if not node_seq:
+            if node_seq is None:
                 continue
 
             link_seq = line['link_sequence']
-            if not link_seq:
+            if link_seq is None:
                 continue
 
             # non-critical info
             agent_id = _convert_str_to_int(line['agent_id'])
-            if not agent_id:
+            if agent_id is None:
                 agent_id = last_agent_id + 1
 
             last_agent_id = agent_id
@@ -582,19 +657,19 @@ def load_columns(ui, input_dir='.'):
                 dp = A.get_demand_period_id(dp)
 
             vol = _convert_str_to_float(line['volume'])
-            if not vol:
+            if vol is None:
                 continue
 
             toll = _convert_str_to_float(line['toll'])
-            if not toll:
+            if toll is None:
                 toll = 0
 
             tt = _convert_str_to_float(line['travel_time'])
-            if not tt:
+            if tt is None:
                 tt = 0
 
             dist = _convert_str_to_float(line['distance'])
-            if not dist:
+            if dist is None:
                 dist = 0
 
             # it could be empty
@@ -734,6 +809,14 @@ def output_columns(ui, output_geometry=True, output_dir='.'):
 
                 writer.writerow(line)
 
+        if output_dir == '.':
+            print('\ncheck agent.csv in '
+                  +os.getcwd()+' for path finding results')
+        else:
+            print('\ncheck agent.csv in '
+                  +os.path.join(os.getcwd(), output_dir)
+                  +' for path finding results')
+
 
 def output_link_performance(ui, output_dir='.'):
     with open(output_dir+'/link_performance.csv', 'w',  newline='') as fp:
@@ -778,6 +861,14 @@ def output_link_performance(ui, output_dir='.'):
 
                 writer.writerow(line)
 
+        if output_dir == '.':
+            print('\ncheck link_performance.csv in '
+                  +os.getcwd()+' for link performance')
+        else:
+            print('\ncheck link_performance.csv in '
+                  +os.path.join(os.getcwd(), output_dir)
+                  +' for link performance')
+
 
 def output_agent_paths(ui, output_geometry=True, output_dir='.'):
     with open(output_dir+'/agent_paths.csv', 'w',  newline='') as f:
@@ -837,4 +928,12 @@ def output_agent_paths(ui, output_geometry=True, output_dir='.'):
                     base.get_agent_link_path(agent_id, True),
                     geometry]
 
-            writer.writerow(line)
\ No newline at end of file
+            writer.writerow(line)
+
+        if output_dir == '.':
+            print('\ncheck agent_paths.csv in '
+                   +os.getcwd()+' for unique agent paths')
+        else:
+            print('\ncheck agent_paths.csv in '
+                  +os.path.join(os.getcwd(), output_dir)
+                  +' for unique agent paths')
\ No newline at end of file
